@@ -6,6 +6,7 @@ import dbus
 import hildon
 
 from procalc import operations
+from procalc.i18n import _
 from procalc.operations import OperationError
 from procalc.stack import OpStack, StackError
 from procalc.helpers import button, switch, picker, selector, liststore, transpose_table
@@ -14,6 +15,10 @@ from procalc.config import Config
 
 class ProCalcApp(hildon.Program):
 
+    __bases = {'Bin': 2, 'Oct': 8, 'Dec': 10, 'Hex': 16, _(u'Auto'): -1}
+    __view_modes = [_(u'Normal'), _(u'Raw'), _(u'Base exp')]
+    __orientations = [_(u'Landscape'), _(u'Portrait'), _(u'Automatic (slider)'), _(u'Automatic (accel)')]
+
     __gsignals__ = {
             'mode-changed': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (int,))
             }
@@ -30,12 +35,12 @@ class ProCalcApp(hildon.Program):
         stack = hildon.TextView()
         input = hildon.Entry(gtk.HILDON_SIZE_AUTO)
 
-        input.set_placeholder('Empty value')
+        input.set_placeholder(_(u'Empty value'))
         input.set_properties(
                 hildon_input_mode=gtk.HILDON_GTK_INPUT_MODE_ALPHA
                 | gtk.HILDON_GTK_INPUT_MODE_NUMERIC
                 | gtk.HILDON_GTK_INPUT_MODE_SPECIAL)
-        stack.set_placeholder('Stack is empty')
+        stack.set_placeholder(_(u'Stack is empty'))
         stack.set_properties(editable=False)
 
         self.w_stack = stack
@@ -156,9 +161,9 @@ class ProCalcApp(hildon.Program):
 
     def create_menu(self):
         menu = hildon.AppMenu()
-        switch(menu, [2, 8, 10, 16].index(self._conv.base), self.hit_switch_base, 'Bin', 'Oct', 'Dec', 'Hex')
+        switch(menu, [2, 8, 10, 16, -1].index(self._conv.base), self.hit_switch_base, *self.__bases.keys())
 
-        menu.append(picker('View mode', (self._conv.mode,), self.hit_change_view, 'Normal', 'Raw', 'Base exp'))
+        menu.append(picker(_(u'View mode'), (self._conv.mode,), self.hit_change_view, *self.__view_modes))
 
         # First number is minimal integer part length to pad to with
         # zeroes (-1 means no leading zero for numbers less than 1),
@@ -166,16 +171,14 @@ class ProCalcApp(hildon.Program):
         # point (0 means always work with integers, -1 - no rounding
         # is applied).
         nums = liststore(*range(-1, 65))
-        menu.append(picker('Precision', map(lambda x: x + 1, self._conv.precision), self.hit_change_precision, selector(nums, nums)))
+        menu.append(picker(_(u'Precision'), map(lambda x: x + 1, self._conv.precision), self.hit_change_precision, selector(nums, nums)))
 
-        menu.append(picker('Orientation', (self.orientation_mode,), self.hit_change_orientation,
-            'Landscape', 'Portrait', 'Automatic (slider)', 'Automatic (accel)'))
-        menu.append(button('About', self.show_about_info))
+        menu.append(picker(_(u'Orientation'), (self.orientation_mode,), self.hit_change_orientation, *self.__orientations))
+        menu.append(button(_(u'About'), self.show_about_info))
         return menu
 
     def show_about_info(self, b):
-        self.note("""
-Programmer's calculator, v0.1
+        self.note(_(u"""Programmer's calculator, v0.1
 
 RPN calculator with bit-wise operations
 and infix operators emulation.
@@ -183,8 +186,7 @@ and infix operators emulation.
 Distributed AS IS under GPLv3 or greater
 without any warranty.
 
-Author: Konstantin Stepanov, © 2010
-                """)
+Author: Konstantin Stepanov, (c) 2010"""))
 
     def filter(self, value):
         return self._conv.format(value)
@@ -247,7 +249,7 @@ Author: Konstantin Stepanov, © 2010
     orientation_mode = property(orientation_mode, set_orientation_mode)
 
     def hit_change_orientation(self, b):
-        new_mode = ['Landscape', 'Portrait', 'Automatic (slider)', 'Automatic (accel)'].index(b.get_value())
+        new_mode = self.__orientations.index(b.get_value())
         self.orientation_mode = new_mode
 
     def hit_execute(self, b):
@@ -277,7 +279,7 @@ Author: Konstantin Stepanov, © 2010
                 self.input = text
                 self.is_mode = False
             else:
-                self.message('Press 2, 8, 0 or A to select base', 2000)
+                self.message(_(u'Press 2, 8, 0 or A to select base'), 2000)
         else:
             self.add_input(b.get_label())
 
@@ -316,8 +318,8 @@ Author: Konstantin Stepanov, © 2010
                 self.message(e.message.capitalize(), 2000)
 
     def hit_switch_base(self, b):
-        base_name = b.get_label()
-        self._conv.base = dict(Bin=2, Oct=8, Dec=10, Hex=16, Auto=-1).get(base_name, 10)
+        base_name = unicode(b.get_label())
+        self._conv.base = self.__bases.get(base_name, 10)
         self.update_view()
 
     def hit_mode(self, b):
@@ -327,13 +329,16 @@ Author: Konstantin Stepanov, © 2010
         self.emit('mode-changed', self.is_mode + (b.get_active() and 2))
 
     def hit_power(self, b):
-        if self.is_mode:
+        if self.is_func:
+            self.input = '[‥]'
+            self.opmode = True
+        elif self.is_mode:
             self.add_input('j')
         else:
             self.add_input('e')
 
     def hit_change_view(self, b):
-        self._conv.mode = ['Normal', 'Raw', 'Base exp'].index(b.get_value())
+        self._conv.mode = self.__view_modes.index(b.get_value())
         self.update_view()
 
     def hit_keyboard(self, w, ev):
@@ -533,12 +538,13 @@ Author: Konstantin Stepanov, © 2010
         self.w_buffer.set_text(self.stack.as_str(self.filter))
 
     def message(self, text, timeout=500):
-        banner = hildon.hildon_banner_show_information(self.window, '', text)
+        banner = hildon.hildon_banner_show_information(self.window, '',
+                str(text))
         banner.set_timeout(timeout)
         return banner
 
     def note(self, text):
-        banner = hildon.hildon_note_new_information(self.window, text)
+        banner = hildon.hildon_note_new_information(self.window, str(text))
         banner.connect('response', lambda s, ev: s.destroy())
         banner.show()
         return banner
@@ -583,25 +589,36 @@ Author: Konstantin Stepanov, © 2010
             buttons_box2.attach(b, 0, 1, i, i + 1)
 
         # Stack operations
-        hooks = [self.hit_push_stack, self.hit_pop_stack]
-        for i, c in enumerate((('st↓', 'dup'), ('st↑', 'st↕'))):
+        hooks = [self.hit_pop_stack, self.hit_push_stack]
+        for i, c in enumerate((('st↑', 'st↕'), ('st↓', 'dup'))):
             b = button(c[0], hooks[i], 'mode')
             b.set_labels(c[0], c[0], c[1], c[1])
             self.connect('mode-changed', b.change_mode)
-            buttons_box1.attach(b, i + 3, i + 4, 4, 5)
+            buttons_box2.attach(b, 2, 3, 3 + i, 4 + i)
 
         # Binary operations
-        for i, c in enumerate(('^', '&~', '&', '~', '|')):
-            b = button(c, self.hit_opkey)
+        for i, c in enumerate((
+            ('^', 'sin', 'sinh', 'asin'),
+            ('&~', 'cos', 'cosh', 'acos'),
+            ('&', 'tan', 'tanh', 'atan'),
+            ('~', 'cot', 'atg2', 'acot'),
+            ('|', '1/x', '%', '%'))):
+            b = button(c[0], self.hit_opkey, 'mode')
+            b.set_labels(*c)
+            self.connect('mode-changed', b.change_mode)
             buttons_box2.attach(b, 1, 2, i, i + 1)
 
-        for i, c in enumerate(('<<', '>>')):
-            b = button(c, self.hit_opkey)
+        for i, c in enumerate((
+            ('<<', 'log', 'π'),
+            ('>>', 'ln', 'e'))):
+            b = button(c[0], self.hit_opkey, 'mode')
+            b.set_labels(c[0], c[1], c[2], c[2])
+            self.connect('mode-changed', b.change_mode)
             buttons_box2.attach(b, 2, 3, i, i + 1)
 
         # Execute
         b = button('=', self.hit_execute)
-        buttons_box2.attach(b, 2, 3, 3, 5)
+        buttons_box1.attach(b, 3, 5, 4, 5)
 
         # Special mode keys
         b = button('Mod', self.hit_mode, 'toggle')
@@ -613,7 +630,7 @@ Author: Konstantin Stepanov, © 2010
         buttons_box1.attach(b, 4, 5, 0, 1)
 
         b = button('×Bⁿ', self.hit_power, 'mode')
-        b.set_labels('×Bⁿ', '+bj', '×Bⁿ', '+bj')
+        b.set_labels('×Bⁿ', '+bj', 'list', 'list')
         self.connect('mode-changed', b.change_mode)
         buttons_box2.attach(b, 2, 3, 2, 3)
 
